@@ -43,7 +43,6 @@ Provides:
 """
 
 from copy import deepcopy
-from math import radians
 import types
 from warnings import warn
 from pathlib import Path
@@ -151,7 +150,7 @@ class SpecificForcingPseudoSpectralCoarse(SpecificForcing):
         ----------
         shape_forcing: array-like
             A single-element array containing index of largest forcing
-            wavenumber or a tuple indincating shape of the forcing array.
+            wavenumber or a tuple indicating shape of the forcing array.
 
         shape: array-like
             A tuple indicating the shape of an array or Operators instance.
@@ -175,9 +174,12 @@ class SpecificForcingPseudoSpectralCoarse(SpecificForcing):
 
         if params.forcing.nkmax_forcing < params.forcing.nkmin_forcing:
             raise ValueError(
-                "params.forcing.nkmax_forcing < \n" "params.forcing.nkmin_forcing"
+                f"params.forcing.nkmax_forcing = {params.forcing.nkmax_forcing} < "
+                f"params.forcing.nkmin_forcing = {params.forcing.nkmin_forcing}"
             )
 
+        # oper.deltak is max deltak_i in the different directions
+        # i.e. based on the smallest edge of the numerical domain.
         self.kmax_forcing = self.oper.deltak * params.forcing.nkmax_forcing
         self.kmin_forcing = self.oper.deltak * params.forcing.nkmin_forcing
 
@@ -189,65 +191,20 @@ class SpecificForcingPseudoSpectralCoarse(SpecificForcing):
             self.key_forced = self._key_forced_default
 
         try:
-            n = 2 * fftw_grid_size(params.forcing.nkmax_forcing)
+            fft_size = 2 * fftw_grid_size(
+                int(round(params.forcing.nkmax_forcing))
+            )
         except ImportError:
             warn("To use smaller forcing arrays: pip install pulp")
             i = 0
-            while 2 * params.forcing.nkmax_forcing > 2 ** i:
+            while 2 * params.forcing.nkmax_forcing > 2**i:
                 i += 1
-            n = 2 ** i
+            fft_size = 2**i
 
-        self._check_forcing_shape([n], sim.oper.shapeX_seq)
-
-        try:
-            angle = self.params.forcing[self.tag].angle
-        except AttributeError:
-            pass
-        else:
-            if isinstance(angle, str):
-                if angle.endswith("°"):
-                    angle = radians(float(angle[:-1]))
-                else:
-                    raise ValueError(
-                        "Angle should be a string with \n"
-                        + "the degree symbol or a float in radians"
-                    )
-
-            self.angle = angle
-            self.kxmax_forcing = np.sin(angle) * self.kmax_forcing
-            self.kymax_forcing = np.cos(angle) * self.kmax_forcing
+        self._check_forcing_shape([fft_size], sim.oper.shapeX_seq)
 
         if mpi.rank == 0:
-            params_coarse = deepcopy(params)
-
-            params_coarse.oper.nx = n
-            # The "+ 1" aims to give some gap between the kxmax and
-            # the boundary of the oper_coarse.
-            try:
-                params_coarse.oper.nx = 2 * fftw_grid_size(
-                    int(self.kxmax_forcing / self.oper.deltakx) + 1
-                )
-            except AttributeError:
-                pass
-
-            try:
-                params_coarse.oper.ny = n
-                try:
-                    params_coarse.oper.ny = 2 * fftw_grid_size(
-                        int(self.kymax_forcing / self.oper.deltaky) + 1
-                    )
-                except AttributeError:
-                    pass
-            except AttributeError:
-                pass
-
-            try:
-                params_coarse.oper.nz = n
-            except AttributeError:
-                pass
-
-            params_coarse.oper.type_fft = "sequential"
-            params_coarse.oper.coef_dealiasing = 1.0
+            params_coarse = self._create_params_coarse(fft_size)
 
             self.oper_coarse = sim.oper.__class__(params=params_coarse)
             self.shapeK_loc_coarse = self.oper_coarse.shapeK_loc
@@ -283,6 +240,25 @@ class SpecificForcingPseudoSpectralCoarse(SpecificForcing):
                 self.shapeK_loc_coarse, root=0
             )
 
+    def _create_params_coarse(self, fft_size):
+        params_coarse = deepcopy(self.sim.params)
+        params_coarse.oper.type_fft = "sequential"
+        params_coarse.oper.coef_dealiasing = 1.0
+
+        params_coarse.oper.nx = fft_size
+
+        try:
+            params_coarse.oper.ny = fft_size
+        except AttributeError:
+            pass
+
+        try:
+            params_coarse.oper.nz = fft_size
+        except AttributeError:
+            pass
+
+        return params_coarse
+
     def _compute_cond_no_forcing(self):
         if hasattr(self.oper_coarse, "K"):
             K = self.oper_coarse.K
@@ -813,11 +789,13 @@ class RandomSimplePseudoSpectral(NormalizedForcing):
         To be called only with proc 0.
         """
         f_fft = self.oper_coarse.create_arrayK_random(min_val=self._min_val)
+        # TODO: adapt to 3D!
         # fftwpy/easypyfft returns f_fft
         f_fft[self.oper_coarse.shapeK_loc[0] // 2] = 0.0
         f_fft[:, self.oper_coarse.shapeK_loc[1] - 1] = 0.0
-        f_fft = self.oper_coarse.project_fft_on_realX(f_fft)
+        #
         f_fft[self.COND_NO_F] = 0.0
+        f_fft = self.oper_coarse.project_fft_on_realX(f_fft)
         return f_fft
 
     def forcingc_raw_each_time(self, a_fft):
@@ -864,8 +842,8 @@ class TimeCorrelatedRandomPseudoSpectral(RandomSimplePseudoSpectral):
                 self._seed1 = int(seed1)
             else:
                 self.t_last_change = self.sim.time_stepping.t
-                self._seed0 = np.random.randint(0, 2 ** 31)
-                self._seed1 = np.random.randint(0, 2 ** 31)
+                self._seed0 = np.random.randint(0, 2**31)
+                self._seed1 = np.random.randint(0, 2**31)
                 self._save_state()
 
             np.random.seed(self._seed0)
@@ -895,7 +873,8 @@ class TimeCorrelatedRandomPseudoSpectral(RandomSimplePseudoSpectral):
             self.t_last_change = tsim
             self._seed0 = self._seed1
             self.forcing0 = self.forcing1
-            self._seed1 = np.random.randint(0, 2 ** 31)
+            self._seed1 = np.random.randint(0, 2**31)
+            np.random.seed(self._seed1)
             self.forcing1 = self.compute_forcingc_raw()
             self._save_state()
 
@@ -903,6 +882,9 @@ class TimeCorrelatedRandomPseudoSpectral(RandomSimplePseudoSpectral):
         return f_fft
 
     def _save_state(self):
+        if not self.params.output.HAS_TO_SAVE:
+            return
+
         with open(self._forcing_state_file_path, "w") as file:
             file.write(
                 "# do not modify by hand\n# t_last_change seed0 seed1\n"
